/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse implementiert einen SBA Anzeigequerschnitt vom Typ 'typ.sswSbaAnzeigeQuerschnitt'.<p>
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class AnzeigeQuerschnittSBA
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static final Debug m_debug = Debug.getLogger();
	/**
   * Schnittstellen, die die logische Verbindung zum Datenverteiler reprsentiert. 
   */
	private ClientDavConnection m_connection;
	/**
	 * Schnittstelle zum Zugriff auf Objekte und Eigenschaften eines Datenmodells.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des SBA Anzeigequerschnitt.
	 */
	private SystemObject m_anzeigeQuerschnittSBA;
	/**
	 * Systemobjekt des Anzeigequerschnitt.
	 */
	private SystemObject m_anzeigeQuerschnitt;
	/**
	 * Betriebskilometer des Anzeigequerschnitts
	 */
	private Betriebskilometer m_betriebskilometer;
	
	/**
	 * Konstruktor mit bernahme der Parameter.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param anzeigeQuerschnitt Systemobjekt Anzeigequerschnitt.
	 */
	public AnzeigeQuerschnittSBA(ClientDavConnection connection, SystemObject anzeigeQuerschnitt)
	{
		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		
		m_anzeigeQuerschnittSBA = anzeigeQuerschnitt;
		bestimmeKonfigurationsDaten(anzeigeQuerschnitt);
	}
	
	/**
	 * Diese Methode bestimmt die Konfigurationsdaten des SBA Anzeigequerschnitts
	 * und legt diese in lokale Variablen ab.
	 * 
	 * @param anzeigeQuerschnitt Objekt Anzeigequerschnitt.
	 */
	private void bestimmeKonfigurationsDaten(SystemObject anzeigeQuerschnitt)
	{
		if (anzeigeQuerschnitt != null)
		{
			if (m_betriebskilometer == null)
			{
				m_betriebskilometer = new Betriebskilometer();
			}
			
			ConfigurationObject co = (ConfigurationObject)anzeigeQuerschnitt;
			if (co != null)
			{
				Data coData = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt"));
				if (coData != null)
				{
					m_anzeigeQuerschnitt = coData.getReferenceValue("AnzeigeQuerschnitt").getSystemObject();
				}

				Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.sswSbaBetriebsKilometer"));
				if (data != null)
				{
					m_betriebskilometer.setKilometer(data.getUnscaledValue("Wert").longValue());
					m_betriebskilometer.setRichtung(data.getScaledValue("Richtung").getValueText());					
				}
			}
		}
		
		return;
	}

	/**
	 * Gibt die Pid des SBA Anzeigequerschnitts zurck.
	 * @return Pid des SBA Anzeigequerschnitts.
	 */
	public String getPid()
	{
		if (m_anzeigeQuerschnittSBA != null)
		{
			return m_anzeigeQuerschnittSBA.getPid();
		}
		return null;
	}

	/**
	 * Gibt die Pid des realen Anzeigequerschnitts zurck.
	 * @return Pid des realen Anzeigequerschnitts.
	 */
	public String getPidAnzeigeQuerschnitt()
	{
		if (m_anzeigeQuerschnitt != null)
		{
			return m_anzeigeQuerschnitt.getPid();
		}
		return null;
	}

	/**
	 * Gibt das Objekt des realen Anzeigequerschnitts zurck.
	 * @return Objekt des realen Anzeigequerschnitts.
	 */
	public SystemObject getObjektAnzeigeQuerschnitt()
	{
		return m_anzeigeQuerschnitt;
	}

	/**
	 * Gibt den Betriebskilometer des Anzeigequerschnitt in Meter zurck.
	 * @return Betriebskilometer in Meter.
	 */
	public long getBetriebskilometer()
	{
		if (m_betriebskilometer != null)
		{
			return m_betriebskilometer.getKilometer();
		}
		return -1;
	}

	/**
	 * Gibt die Richtung der Strae zurck.
	 * @return Richtung der Strae.
	 */
	public String getRichtung()
	{
		if (m_betriebskilometer != null)
		{
			return m_betriebskilometer.getRichtung();
		}
		return null;
	}

	/**
	 * Diese Klasse dient zum Speichern eines Betriebskilometers.
	 *
	 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	private class Betriebskilometer
	{
		/**
		 * Betriebsilometer in Meter.
		 */
		private long m_kilometer;
		/**
		 * Richtung.
		 */
		private String m_richtung;
		
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
		public Betriebskilometer()
		{			
		}

		/**
		 * Gibt den Betriebsilometer in Meter zurck.
		 * @return Betriebsilometer in Meter.
		 */
		public long getKilometer()
		{
			return m_kilometer;
		}

		/**
		 * Setzt den Betriebsilometer in Meter.
		 * @param kilometer Wert auf den der Betriebsilometer gesetzt werden soll.
		 */
		public void setKilometer(long kilometer)
		{
			m_kilometer = kilometer;
		}

		/**
		 * Gibt die Richtung der Strae zurck.
		 * @return Richtung der Strae.
		 */
		public String getRichtung()
		{
			return m_richtung;
		}

		/**
		 * Setzt die Richtung der Strae.
		 * @param richtung Wert auf den die Richtung gestzt werden soll.
		 */
		public void setRichtung(String richtung)
		{
			m_richtung = richtung;
		}
	}
}
