/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.stau;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.ste.stealgsba.algorithmen.lib.Sequenz;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.AlgorithmusrahmenVollausbau;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.EinzelMassnahme;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenanforderung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenbearbeitung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Ursacheneinheit;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IAlgorithmus;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IMassnahmenbearbeitung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.ISituationsbearbeitung;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IUrsacheneinheitenListener;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert den Steuerungsalgorithmus Stau.<p>
 *
 * Dieser Steuerungsalgorithmus erkennt Stausituationen und sichert diese in stromaufwrtiger Richtung ab.
 * Zugleich werden im Staubereich Geschwindigkeitsbeschrnkungen und andere unpassende Schaltungen
 * unterdrckt. Der Steuerungsalgorithmus kann auch zur Absicherung von Staus auf Ausfahrtsrampen
 * verwendet werden.
 *
 * @version $Revision: 1.3 $ / $Date: 2015/03/03 14:26:00 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class StauAlg extends AlgorithmusrahmenVollausbau implements IAlgorithmus, ISituationsbearbeitung, IMassnahmenbearbeitung, IUrsacheneinheitenListener, ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Schnittstellen, die die logische Verbindung zum Datenverteiler reprsentiert. 
   */
	private ClientDavInterface m_connection;
	/**
	 * Schnittstelle zum Zugriff auf Objekte und Eigenschaften eines Datenmodells.
	 */
	private DataModel m_dataModel;
	/**
	 * Typ der zu bearbeitenden Ursacheneinheit.
	 */
	private final String m_typUrsacheneinheit = "typ.sswSbaUeStau";
	/**
	 * Liste der Ursacheneinheiten.
	 */
	private List<Ursacheneinheit> m_ursacheneinheiten = null;
	/**
	 * Attributgruppe fr den Empfang der Fahrstreifenpassivierung.
	 */
	private AttributeGroup m_atgPassivierungFahrstreifen;
	/**
	 * Aspekt zum Versand der Manahmenanforderung.
	 */
	private Aspect m_aspMassnahmeanforderung;	
  /**
   * Datensatz der Parameter passivierte Fahrstreifen.
   */
  private ResultData m_passivierungFahrStreifen;
	/**
	 * Objekt zur Synchornisation.
	 */
	private Object m_sync;	
	/**
	 * Objekt fr die Massnahmenbearbeitung.
	 */
	private Massnahmenbearbeitung m_mab;
	/**
	 * Aspekt fr den Empfang .
	 */
	private Aspect m_aspPassivierungFahrstreifen;	
	/**
	 * Flag, neue Daten ber die Fahrstreifenpassivierung erhalten.
	 */
	private boolean m_neueFahrstreifenPassivierungErhalten = false;
	/**
	 * bergabeparameter -verknuepfungMqAlsUnd=true/false</br>
	 * Defaultwert=false
	 */
	private boolean m_verknuepfungMqAlsUnd;
	
	/**
	 * Konstruktor.<p>
	 * 
	 * Festlegung, fr welche Typen von Ursacheneineheiten der Algorithmus arbeiten soll.
	 * Instanziierung des Objekts fr die Massnahmenbearbeitung.
	 */
	public StauAlg()
	{
		m_ursacheneinheitTypen.add(m_typUrsacheneinheit);
		
		m_mab = new Massnahmenbearbeitung(this, this);
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.AlgorithmusrahmenGrundausbau#parseArguments(de.bsvrz.sys.funclib.commandLineArgs.ArgumentList)
	 */
	@Override
	public void parseArguments(ArgumentList argumentList) throws Exception
	{
		super.parseArguments(argumentList);

		m_debug = Debug.getLogger();
		
		m_verknuepfungMqAlsUnd = argumentList.fetchArgument("-verknuepfungMqAlsUnd=false").booleanValue();		
	  m_debug.info("VerknuepfungMqAlsUnd= " + m_verknuepfungMqAlsUnd);
	}

	/**
	 * Thread zur Bearbeitung der Manahmeanforderungen.
	 *
	 * @version $Revision: 1.3 $ / $Date: 2015/03/03 14:26:00 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Mattihas Obert
	 *
	 */
	private class StauThread implements Runnable
	{
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
		public StauThread()
		{			
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run()
		{
			List<Massnahmenanforderung> mas = new ArrayList<Massnahmenanforderung>();
			
			while (true)
			{
				try
				{
					/**
					 * Warten bis Methode 'starteBerechnung' fertig.
					 */
					synchronized (m_sync)
					{
						m_sync.wait();
					}
				}
				catch (InterruptedException e1)
				{
					e1.printStackTrace();
				}
				
				System.out.println("StauThread.run() -> Massnahmebearbeitung");

				/**
				 * Massnahmebearbeitung
				 */

//				List<Massnahmenanforderung> mas = m_mab.erzeugeMassnahmenanforderungen();
				mas.clear();
				mas.addAll(m_mab.erzeugeMassnahmenanforderungen());

				for (Massnahmenanforderung massnahmenanforderung : mas)
				{
					if (massnahmenanforderung != null)
					{						
						massnahmenanforderung.setAbsenderAlgorithmus("Stau");
						massnahmenanforderung.setAbsenderApplikation(getConnection().getLocalApplicationObject().getId());
						massnahmenanforderung.setNummer(Sequenz.next());
						
						massnahmenanforderung.setSpontan(isSpontan());
						massnahmenanforderung.setLoeschen(false);

						Ursacheneinheit ue = getUrsacheneinheit(massnahmenanforderung.getReferenz());
						if (ue != null)
						{
							if (ue.isBearbeitung())
							{
							  versendeMassnahmenanforderungAspekt(massnahmenanforderung, ue.getAspektVerwendung());
							}
							else
							{
								loescheMassnahmenanforderung(massnahmenanforderung);
							}
						}						
					}
				}
			}
		}
	}

	/**
	 * Statische Methode fr das Starten des Programms.
	 * 
	 * @param args Kommandozeilen-Parameter
	 */
	public static void main(String[] args)
	{
		StandardApplicationRunner.run(new StauAlg(), args);
	}


	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.AlgorithmusrahmenGrundausbau#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	@Override
	public void initialize(ClientDavInterface connection) throws Exception
	{
		super.initialize(connection);

		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		
		Sequenz.set(m_connection.getTime());

		m_sync = new Object();
		m_aspMassnahmeanforderung = getAspektVersand();
			
		this.setzeCallbackObjekt(this);
		
		System.out.println("StauAlg.initialize()");

		try
		{
			m_atgPassivierungFahrstreifen = m_dataModel.getAttributeGroup("atg.sswSbaPassivierungFahrStreifen");
			m_aspPassivierungFahrstreifen = m_dataModel.getAspect("asp.sswSbaPassivierung");

			empfaengerAnmeldenObjekt(this, m_dataModel.getObject(gibAnlage()), m_atgPassivierungFahrstreifen, m_aspPassivierungFahrstreifen);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		/**
		 * Thread starten
		 */	  
		Thread algorithmus = new Thread(new StauThread());
		algorithmus.setName(this.getClass().getName() + " " + new Date());
		algorithmus.start();
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.interfaces.IAlgorithmus#starteBerechnung()
	 */
	public synchronized void starteBerechnung()
	{
		System.out.println("StauAlg.starteBerechnung()");
		
		if (m_ursacheneinheiten != null)
		{
			/**
			 * Neue Daten bzgl. der Fahrstreifenpassivierung an die Ursacheneinheiten weitergeben.
			 */
			if (m_neueFahrstreifenPassivierungErhalten)
			{
				m_neueFahrstreifenPassivierungErhalten = false;

				updateDatenUrsacheneinheiten();
			}

			/**
			 * Situationsbearbeitung aller Ursacheneinheiten
			 */
			for (int i = 0; i < m_ursacheneinheiten.size(); i++)
			{
				StauUe ueAlg = (StauUe)m_ursacheneinheiten.get(i);
				
				if (ueAlg != null)
				{
					ueAlg.setZyklusLaenge(getZyklusLaenge());

					setzeStatus(ueAlg.berechnen(), (Ursacheneinheit)ueAlg);

					sendeUrsacheneinheitStatus(ueAlg);
				}
			}

			// Berechnung der Manahmenanforderung einleiten.
			synchronized (m_sync)
			{
				m_sync.notify();
			}		
		}
	}

	/**
	 * Gibt das Objekt Ursacheneinheit fr das angegebene SystemObjekt zurck.
	 * 
	 * @param objUe SystemObjekt der Ursacheneinheit.
	 * @return Objekt Ursacheneinheit oder <code>null</code>.
	 */
	private Ursacheneinheit getUrsacheneinheit(SystemObject objUe)
	{
		if (m_ursacheneinheiten != null)
		{
			for (int i = 0; i < m_ursacheneinheiten.size(); i++)
			{
				Ursacheneinheit ue = m_ursacheneinheiten.get(i);
				
				if (ue != null)
				{
					if (ue.getObjekt().equals(objUe))
						return ue;
				}
			}
		}
		return null;
	}

	/**
	 * Diese Methode aktualisiert fr alle Ursacheneinheiten die Daten der Fahrstreifenpassivierung.
	 */
	private void updateDatenUrsacheneinheiten()
	{
		System.out.println("StauAlg.updateDatenUrsacheneinheiten()");
		
		if (m_ursacheneinheiten != null)
		{
			for (int i = 0; i < m_ursacheneinheiten.size(); i++)
			{
				StauUe ue = (StauUe)m_ursacheneinheiten.get(i);
				
				if (ue != null)
				{
					ue.updatePassivierungFahrstreifen(m_passivierungFahrStreifen);
				}
			}
		}
	}

	/**
	 * Aktualisiert die interne Liste der Ursacheneinheiten.
	 */
	private void updateUrsachenEinheiten()
	{
		m_ursacheneinheiten = gibUrsacheneinheitenTyp(m_typUrsacheneinheit);
		
		System.out.println("Zu bearbeitenden UE's " + m_ursacheneinheiten);
	}


	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.interfaces.IUrsacheneinheitenListener#objektUrsacheneinheitErstellen(de.bsvrz.dav.daf.main.config.DynamicObject)
	 */
	public Ursacheneinheit objektUrsacheneinheitErstellen(DynamicObject obj)
	{
		if (obj != null)
		{
			SystemObjectType typ = obj.getType();
			
			if (typ.getPid().equals(m_typUrsacheneinheit))
			{
				StauUe ue = new StauUe(getConnection(), (SystemObject)obj, getAspektVersand(), m_verknuepfungMqAlsUnd);
				ue.addListener(this);
				ue.datenAnmelden();
				
				System.out.println("UE " + ue);
				
				return (Ursacheneinheit)ue;
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.AlgorithmusrahmenVollausbau#updateEinzelMassnahme(de.bsvrz.ste.algorithmusrahmen.Ursacheneinheit, de.bsvrz.ste.algorithmusrahmen.EinzelMassnahme)
	 */
	@Override
	public void updateEinzelMassnahme(Ursacheneinheit ue, EinzelMassnahme einzelMassnahme)
	{
		super.updateEinzelMassnahme(ue, einzelMassnahme);

		System.out.println("StauAlg.updateEinzelMassnahme() " + ue);
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.interfaces.IUrsacheneinheitenListener#updateUrsachenEinheiten(de.bsvrz.ste.algorithmusrahmen.Ursacheneinheit[], de.bsvrz.ste.algorithmusrahmen.Ursacheneinheit[])
	 */
	public void updateUrsachenEinheiten(List<Ursacheneinheit> addedUrsacheneinheiten, List<Ursacheneinheit> removedUrsacheneinheiten)
	{
		System.out.println("StauAlg.updateUrsachenEinheiten()");
		
		updateUrsachenEinheiten();
	}
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  public boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData results[])
	{
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];

			if (resultData.getDataDescription().getAttributeGroup().equals(m_atgPassivierungFahrstreifen))
	  	{ 
				m_passivierungFahrStreifen = resultData;
				
				m_neueFahrstreifenPassivierungErhalten = true;
	  		
//				System.out.println("PassivierungFahrstreifen -> " + m_passivierungFahrStreifen);	
	  	}    	
		}
	}

}
